#include <gint/gray.h>
#include <gint/keyboard.h>
#include <gint/timer.h>
#include <stdbool.h>
#include <gint/clock.h>
#include <gint/defs/util.h>

struct Line {
	unsigned char pos;
	unsigned char length;
};

int callback(volatile int *flag)
{
	*flag = 1;

	return TIMER_CONTINUE;
}

void render(struct Line *screen, bool win)
{
	extern bopti_image_t img_logo, img_win;

	dclear(C_WHITE);

	dimage(121, 14, &img_logo);

	for(int i = 0; i < 8; i++)
	{
		dline(0, i * 8 + 3, 119, i * 8 + 3, C_LIGHT);
	}
	for(int i = 1; i < 16; i++)
	{
		dline(i * 8 - 1, 4, i * 8 - 1, 59, C_LIGHT);
	}

	for(int row = 0; row < 15; row++)
	{ 
		for(int column = screen[row].pos; column < screen[row].pos + screen[row].length; column++)
		{
			drect(row * 8, column * 8 + 4, row * 8 + 6, column * 8 + 10, C_BLACK);
		}
	}

	if(win) dimage(59, 14, &img_win);

	dupdate();

	return;
}

int update(struct Line *screen, int *row, int *direction, int frames)
{
	key_event_t key;
	int pos;
	int overlap;

	if(frames % (19 - *row) == 0)
	{
		if(*direction == 0 && screen[*row].pos == 0) *direction = 1;
		if(*direction == 1 && screen[*row].pos + screen[*row].length == 7) *direction = 0;

		if(*direction == 0) screen[*row].pos--;
		else screen[*row].pos++;
	}

	key = pollevent();
	while(key.type != KEYEV_NONE)
	{
		if(key.key == KEY_MENU) return -1;

		if(key.key == KEY_EXE && key.type == KEYEV_DOWN)
		{
			if(*row == 0)
			{
				pos = screen[0].pos;
				overlap = 3;
			}
			else
			{
				pos = max(screen[*row].pos, screen[*row - 1].pos);
				overlap = min(screen[*row].pos + screen[*row].length, screen[*row - 1].pos + screen[*row - 1].length) - pos;
			}
			if(*row == 14 && overlap > 0) return 2;
			if(overlap <= 0) return 0;
			screen[*row].pos = pos;
			screen[*row].length = overlap;
			(*row)++;
			screen[*row] = screen[*row - 1];
			if(*row == 8 && screen[*row].length == 3) screen[*row].length = 2;
			if(*row == 11 && screen[*row].length == 2) screen[*row].length = 1;
		}

		key = pollevent();
	}

	return 1;
}

int main(void)
{
	struct Line screen[15];
	int row;
	int direction;
	int timer;
	volatile int flag = 0;
	int frames;
	int ret;
	
	dgray(DGRAY_ON);
	
	timer = timer_setup(TIMER_ANY, 12 * 1000, &callback, &flag);
	timer_start(timer);

	while(true)
	{
		for(int i = 0; i < 15; i++) screen[i] = (struct Line){ 0 };
		screen[0] = (struct Line){0, 3};
		row = 0;
		direction = 1;
		frames = 0;

		while(true)
		{
			ret = update(screen, &row, &direction, frames);
			if(ret != 1) break;
			render(screen, false);
			frames++;

			while(!flag) sleep();
			flag = 0;
		}

		if(ret == 2)
		{
			render(screen, true);
			getkey();
		}

		if(ret == -1) break;
	}

	timer_stop(timer);

	return 1;
}
